//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "Console.hpp"

//------------------------------------------------------------------------------
#include "Global.hpp"
#include "config.hpp"
#include "lua.hpp" // To get lua version

#include "Utils.hpp"
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
using namespace std;
//------------------------------------------------------------------------------
ConsoleStreamBuf::ConsoleStreamBuf(Console *targetConsole, streambuf *targetCopyBuf, bool isError) : m_targetConsole(targetConsole), m_targetCopyBuf(targetCopyBuf), m_isError(isError)
{
    // Initialize get pointer.  This should be zero so that underflow is called upon first read.
    setg(0, 0, 0);

    // Initialize the put pointer.  Overflow won't get called until this buffer is filled up,
    // so we need to use valid pointers.
    setp(m_buffer, m_buffer + BUFFER_SIZE - 1);
}
//------------------------------------------------------------------------------
int ConsoleStreamBuf::overflow(int c)
{
    char* begin = pbase();
    char* end = pptr();
    *end = '\0'; // Add an end of line char to make a string

    // Copy the buffer to the targets
    m_targetCopyBuf->sputn(begin, end - begin);
    m_targetConsole->AddText(begin, m_isError);

    // Tell that the buffer is now free
    setp(m_buffer, m_buffer + BUFFER_SIZE - 1);

    return traits_type::not_eof(c);
}
//------------------------------------------------------------------------------
int ConsoleStreamBuf::sync()
{
    return traits_type::eq_int_type(this->overflow(traits_type::eof()), traits_type::eof()) ? -1 : 0;
}
//------------------------------------------------------------------------------
Console::Console() : m_interLine(5), m_heightRatio(0.4), m_margin(3), m_isVisible(false)
{
    // text & font
    m_text.SetSize(13);
    if(m_font.LoadFromFile(Global::GetCommonDataPath() + "/console.ttf")) // Segfault if file doesn't exist on Linux
    {
        m_text.SetFont(m_font);
    }

    // The font width should be fixed
    m_text.SetText("a");
    sf::FloatRect rect = m_text.GetRect();
    m_letterSize.x = rect.GetWidth();
    m_letterSize.y = rect.GetHeight();

    // lines
    m_lines.push_back(pair<string, bool>("", false)); // We need a first line

    // We redirect cout and cerr only when everythong is ready
    // Cout & cerr
    m_cerrBuf = const_cast<std::streambuf*>(cerr.rdbuf());
    m_coutBuf = const_cast<std::streambuf*>(cout.rdbuf());

    m_consoleBufCerr = new ConsoleStreamBuf(this, m_cerrBuf, true);
    m_consoleBufCout = new ConsoleStreamBuf(this, m_coutBuf, false);

    cerr.rdbuf(m_consoleBufCerr);
    cout.rdbuf(m_consoleBufCout);
}
//------------------------------------------------------------------------------
Console::~Console()
{
    cerr.rdbuf(m_cerrBuf);
    cout.rdbuf(m_coutBuf);

    delete m_consoleBufCerr;
    delete m_consoleBufCout;
}
//------------------------------------------------------------------------------
void Console::AddText(const char *ptr, bool isError)
{
    string str(ptr);

    size_t newLinePos = -1;

    while(1)
    {
        size_t newLinePosBack = newLinePos + 1; // We skip the found \n
        newLinePos = str.find('\n', newLinePosBack);
        if(newLinePos == string::npos)
        {
            m_lines.back().first += str.substr(newLinePosBack);
            break;
        }
        else
        {
            m_lines.back().first += str.substr(newLinePosBack, newLinePos - newLinePosBack);
            m_lines.back().second = isError;
            m_lines.push_back(pair<string, bool>("", false)); // We insert a new line for the next lines
        }
    }

    if(m_lines.size() > CONSOLE_MAX_LINES)
    {
        for(size_t i = 0; i < CONSOLE_LINE_MAX_RM; i++)
        {
            m_lines.pop_front();
        }
    }
}
//------------------------------------------------------------------------------
void Console::Show(bool isVisible)
{
    m_isVisible = isVisible;
}
//------------------------------------------------------------------------------
void Console::Render(sf::RenderWindow &App)
{
    // Text
    if(! (m_isVisible && App.IsOpened()))
    {
        return;
    }

    const sf::View &backView = App.GetView();
    int width = App.GetWidth();
    int height = App.GetHeight();
    sf::View tmpView(sf::FloatRect(0, 0, width, height));
    App.SetView(tmpView);

    App.Draw(sf::Shape::Rectangle(0.f, 0.f, width, height * m_heightRatio, sf::Color(0, 0, 0, 200), 2, sf::Color(0, 150, 0)));

    size_t maxLineSize = (width - 2.f * m_margin) / m_letterSize.x;
    maxLineSize = max(maxLineSize, size_t(1)); // Can't be 0
    bool isTop = false;

    float currentHeight = height * m_heightRatio - m_margin;
    for(ListType::const_reverse_iterator it = ++m_lines.rbegin(); !isTop && it != m_lines.rend(); it++)
    {
        // Text color
        if(it->second) // error message
        {
            m_text.SetColor(sf::Color(255, 0, 0));
        }
        else
        {
            m_text.SetColor(sf::Color(50, 255, 50));
        }

        int start = (it->first.size() / maxLineSize) * maxLineSize;

        for(int curentPos = start; curentPos >= 0 && !isTop; curentPos -= maxLineSize)
        {
            currentHeight-= m_letterSize.x + m_interLine;
            m_text.SetPosition(m_margin, currentHeight);
            m_text.SetText(StrToSfUtf8(it->first.substr(curentPos, maxLineSize)));
            App.Draw(m_text);

            if(currentHeight < 0)
            {
                isTop = true;
            }
        }
    }

    // Header
    m_text.SetColor(sf::Color(255, 255, 255));
    App.Draw(sf::Shape::Rectangle(0.f, 0.f, width, m_letterSize.y + m_margin, sf::Color(0, 0, 0, 255), 0));
    m_text.SetPosition(Vector2D(m_margin, 0.f));
    stringstream msg, modeInfoStr;
    string modId = Global::GetConfigManager().GetCurrentModeId();
    if(modId == "None")
    {
        modeInfoStr << modId;
    }
    else
    {
        const ModeInfo &info = Global::GetModeInfo();
        modeInfoStr << info.idName << " " << info.version;
    }
    msg << PACKAGE_NAME << " " << VERSION << " / " << LUA_RELEASE << " / Mode: " << modeInfoStr.str();
    m_text.SetText(StrToSfUtf8(msg.str()));
    App.Draw(m_text);

    msg.str("");
    msg.precision(2);
    msg << "FPS: " << fixed << 1.f / App.GetFrameTime();
    m_text.SetPosition(Vector2D(width - m_margin - float(msg.str().size()) * m_letterSize.x, 0.f));
    m_text.SetText(StrToSfUtf8(msg.str()));
    App.Draw(m_text);

    App.SetView(backView);
}
//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
