require("guiLib/common")

------------------------------------------------------------
-- Create a button control
------------------------------------------------------------
function buttonControlCreate(group, label, size)
	group:SetUserData("onClickFn", nil)
	group:SetUserData("onOverFn", nil)
	group:SetUserData("isClicking", false)
	group:SetUserData("imagePack", nil)
	group:SetUserData("isEnabled", true)
	group:SetUserData("onClickSound", nil)

	-- Controls
	local tmp = group:CreateSprite("back")
	local zone = group:CreatePhRect(0, "zone", size)
	local text = group:CreateText("text", label)

	-- Events
	local screenOfGroup = group:GetParentScreen()
	
	local zoneEvent = screenOfGroup:RegisterMouseZoneEvent(buttonControlApi_onMenuOver, zone)
	guiLib_addAutoDestroyEvent(group, zoneEvent)
	
	local mouseClickEvent = screenOfGroup:RegisterButtonInZoneEvent(buttonControlApi_onMouseClick, zoneEvent, "MouseLeft", true)
	guiLib_addAutoDestroyEvent(group, mouseClickEvent)
	
	local mouseClickEvent = screenOfGroup:RegisterButtonInZoneEvent(buttonControlApi_onMouseClick, zoneEvent, "MouseLeft", false)
	guiLib_addAutoDestroyEvent(group, mouseClickEvent)
	
	-- Size
	buttonSetCurrentStyle(group, 1)
	buttonControlSetSize(group, size)
end

------------------------------------------------------------
-- Set on click sound
------------------------------------------------------------
function buttonSetClickSound(group, sound)
	group:SetUserData("onClickSound", sound)
end

------------------------------------------------------------
-- Set the label of the button
------------------------------------------------------------
function buttonSetLabel(group, label)
	group:FindByNameText("text"):SetText(label)
	buttonControlSetSize(group, group:FindByNameSprite("back"):GetSize())
end

------------------------------------------------------------
-- Get the label of the button
------------------------------------------------------------
function buttonGetLabel(group)
	return group:FindByNameText("text"):GetText()
end

------------------------------------------------------------
-- Set the text size of a button
------------------------------------------------------------
function buttonSetFontSize(group, size)
	group:FindByNameText("text"):SetSize(size)
	
	buttonControlSetSize(group, buttonControlGetSize(group))
end

------------------------------------------------------------
-- Set the font (size + color + font)
------------------------------------------------------------
function buttonSetFont(group, size, color, font)
	local text = group:FindByNameText("text")

	if size then
		text:SetSize(size)
	end
	
	if color then
		text:SetColor(color)
	end
	
	if font then
		text:SetFont(font)
	end
	
	buttonControlSetSize(group, buttonControlGetSize(group))
end

------------------------------------------------------------
-- Set image pack => Set the background images
------------------------------------------------------------
function buttonSetImagePack(group, normal, over, clicking)
	local tbl = {}
	
	if over == nil then
		over = normal
	end
	
	if clicking == nil then
		clicking = over
	end
	
	table.insert(tbl, normal)
	table.insert(tbl, over)
	table.insert(tbl, clicking)
	
	group:SetUserData("imagePack", tbl)
	buttonSetCurrentStyle(group, 1)
end
	

------------------------------------------------------------
-- Resize a button control
------------------------------------------------------------
function buttonControlSetSize(group, size)
	group:FindByNamePhRect("zone"):SetSize(size)
	
	local text = group:FindByNameText("text")
	local txtSize = text:GetTextSize()
	
	text:SetPosition(Vector2D(size.x / 2 - txtSize.x / 2, size.y / 2 - txtSize.y / 2 - 2))
	
	group:FindByNameSprite("back"):SetSize(size)
end

------------------------------------------------------------
-- Get size of a button control
------------------------------------------------------------
function buttonControlGetSize(group)
	return group:FindByNamePhRect("zone"):GetSize()
end

------------------------------------------------------------
-- Set click event function
------------------------------------------------------------
function buttonSetEventFunction(group, fn)
	if type(fn) == "function" then
		group:SetUserData("onClickFn", fn)
	else
		group:SetUserData("onClickFn", nil)
	end
end

------------------------------------------------------------
-- Set over event function
------------------------------------------------------------
function buttonSetOverFunction(group, fn)
	if type(fn) == "function" then
		group:SetUserData("onOverFn", fn)
	else
		group:SetUserData("onOverFn", nil)
	end
end

------------------------------------------------------------
-- Enable / disable the button
------------------------------------------------------------
function buttonEnable(group, isEnabled)
	group:SetUserData("isEnabled", isEnabled)
	
	if isEnabled == false then
		buttonSetCurrentStyle(group, 1) -- Set nomal image
	end
end

------------------------------------------------------------
-- Set the current style of the button (1 = normal, 2 = over, 3 = clicking)
------------------------------------------------------------
function buttonSetCurrentStyle(group, styleId)
	local imagePack = group:GetUserData("imagePack")
	local spr = group:FindByNameSprite("back")
	
	if imagePack ~= nil and imagePack[styleId] ~= nil then
		spr:SetImage(imagePack[styleId])
		spr:SetSize(buttonControlGetSize(group))
		spr:Show(true)
	else
		spr:Show(false)
	end
end

------------------------------------------------------------
-- Events
------------------------------------------------------------
-- Mouse over
function buttonControlApi_onMenuOver(zoneEvent)
	local group = zoneEvent:GetZone():GetParent();
	
	if not group:GetUserData("isEnabled") then
		-- Event over même si disabled
		if not windowIsPointHidden(windowGetParentWindow(group), GetMousePosition()) then
			local fn = group:GetUserData("onOverFn")
			if fn ~= nil then
				fn(group, zoneEvent:IsInside())
			end
		end
		
		-- End fn		
		return
	end

	local textObj = group:FindByNameText("text")

	if not windowIsPointHidden(windowGetParentWindow(group), GetMousePosition()) then
		local inside = zoneEvent:IsInside()
		
		if inside then
			buttonSetCurrentStyle(group, 2) -- Over
		else
			buttonSetCurrentStyle(group, 1) -- Normal
			group:SetUserData("isClicking", false)
		end
		
		local fn = group:GetUserData("onOverFn")
		if fn ~= nil then
			fn(group, inside)
		end
	else
		buttonSetCurrentStyle(group, 1) -- Normal
	end
end

-- Mouse click
function buttonControlApi_onMouseClick(keyName, pressed, zoneEvent, event)
	local group = zoneEvent:GetZone():GetParent()

	if not group:GetUserData("isEnabled") then
		return
	end

	if zoneEvent:IsInside() and windowIsWinParentActive(group) and not windowIsPointHidden(windowGetParentWindow(group), GetMousePosition()) then
		local textObj = group:FindByNameText("text")
	
		if pressed then
			buttonSetCurrentStyle(group, 3) -- Pressed
			
			-- To activate the button, we need to press the mouse in the zone, and to release it in the zone too
			group:SetUserData("isClicking", true)
		else
			-- We call the event function
			if group:GetUserData("isClicking") then
				group:SetUserData("isClicking", false)
				
				buttonSetCurrentStyle(group, 2) -- Over
				
				local fn = group:GetUserData("onClickFn")
				if fn ~= nil then
					if group:GetUserData("onClickSound") ~= nil then
						GetGlobalSoundManager():PlaySound(sndBtClick)
					end
					fn(group)
				end
			end
		end
	end
end

